----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:51:40 04/11/2010 
-- Design Name: 
-- Module Name:    neuron - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity neuron is
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           ce : in  STD_LOGIC;
           y_i : in  STD_LOGIC_VECTOR (31 downto 0);
           i : in  STD_LOGIC_VECTOR (7 downto 0);
			  w_ij_in : in  STD_LOGIC_VECTOR (31 downto 0);
			  we : in STD_LOGIC;
           y_j : out  STD_LOGIC_VECTOR (15 downto 0);
           oe : out  STD_LOGIC);
end neuron;

architecture Behavioral of neuron is

component neuron_state is
	Port ( clk    : in STD_LOGIC;
			  enable : in  STD_LOGIC;
			  reset  : in  STD_LOGIC;
           switch_enable : out  STD_LOGIC;
			  sigmoid_enable : out  STD_LOGIC);
end component;

component switch is
    Port ( clk : in  STD_LOGIC;
           ce : in  STD_LOGIC;
			  y_i_in : in  STD_LOGIC_VECTOR (31 downto 0);
			  i_in   : in  STD_LOGIC_VECTOR (7 downto 0);
			  w_ij_in : in  STD_LOGIC_VECTOR (31 downto 0);
			  we : in STD_LOGIC;
           y_i_out : out  STD_LOGIC_VECTOR (31 downto 0);
           w_ij : out  STD_LOGIC_VECTOR (31 downto 0);
			  oe : out STD_LOGIC);
end component;

component mult is 
	Port ( clk : in  STD_LOGIC;
			  ce  : in  STD_LOGIC;
			  y_i : in  STD_LOGIC_VECTOR (31 downto 0);
           w_ij : in  STD_LOGIC_VECTOR (31 downto 0);
           x_ij : out  STD_LOGIC_VECTOR (63 downto 0);
			  oe : out  STD_LOGIC);
end component;

component accumulator is
    Port ( clk : in  STD_LOGIC;
			  ce  : in  STD_LOGIC;
           clr : in  STD_LOGIC;
			  x_ij : in  STD_LOGIC_VECTOR (31 downto 0);
           x_j : out  STD_LOGIC_VECTOR (31 downto 0);
			  oe : out STD_LOGIC);
end component;

component sigmoid is
    Port ( clk 	: in   STD_LOGIC;
           ce     : in   STD_LOGIC;
			  x 		: in   STD_LOGIC_VECTOR (31 downto 0);
           y      : out  STD_LOGIC_VECTOR (15 downto 0);
			  oe     : out  STD_LOGIC);
end component;

signal switch_enable: STD_LOGIC;
signal sigmoid_enable: STD_LOGIC;
signal y_i_state: STD_LOGIC_VECTOR (31 downto 0);
signal i_state: STD_LOGIC_VECTOR (7 downto 0);
signal y_i_out: STD_LOGIC_VECTOR (31 downto 0);
signal w_ij: STD_LOGIC_VECTOR (31 downto 0);
signal mult_enable: STD_LOGIC;
signal x_ij: STD_LOGIC_VECTOR (63 downto 0);
signal x_j: STD_LOGIC_VECTOR (31 downto 0);
signal accumulator_enable: STD_LOGIC;
signal oe_i : STD_LOGIC;
signal accumulator_clear: STD_LOGIC;
signal accumulator_oe:STD_LOGIC;
signal sigmoid_enable_i:STD_LOGIC;

begin
	
	accumulator_clear <= oe_i or reset;
	sigmoid_enable <= sigmoid_enable_i and not accumulator_oe and not reset;
	
	U0: neuron_state port map(clk, ce, reset, switch_enable, sigmoid_enable_i);
	U1: switch port map(clk, switch_enable, y_i, i, w_ij_in, we, y_i_out, w_ij, mult_enable);
	U2: mult port map(clk, mult_enable, y_i_out, w_ij, x_ij, accumulator_enable);
	U3: accumulator port map(clk, accumulator_enable, accumulator_clear, x_ij(63 downto 32), x_j, accumulator_oe);
	U4: sigmoid port map(clk, sigmoid_enable, x_j, y_j, oe_i);
	
	oe <= oe_i;

end Behavioral;

