----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:30:08 04/10/2010 
-- Design Name: 
-- Module Name:    neuron_state - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity neuron_state is
    Port ( clk    : in STD_LOGIC;
			  enable : in  STD_LOGIC;
			  reset  : in  STD_LOGIC;
           switch_enable : out  STD_LOGIC;
			  sigmoid_enable : out  STD_LOGIC);
end neuron_state;

architecture Behavioral of neuron_state is

type state_type is (st1_idle, st2_read, st3_done);
signal state, next_state : state_type;
signal switch_enable_i : std_logic;
signal sigmoid_enable_i : std_logic;

begin

	SYNC_PROC: process(clk)
	begin
	
		if(clk'event and clk='1') then
			if(reset='1') then
				state <= st1_idle;
				switch_enable <= '0';
				sigmoid_enable <= '0';
			else
				state <= next_state;
				switch_enable <= switch_enable_i;
				sigmoid_enable <= sigmoid_enable_i;
			end if;
		end if;
	
	end process;
	
	OUTPUT_DECODE: process (state)
	begin
		case (state) is
			when st1_idle =>
				switch_enable_i <= '0';
				sigmoid_enable_i <= '0';
			when st2_read =>
				switch_enable_i <= '1';
				sigmoid_enable_i <= '0';
			when st3_done =>
				switch_enable_i <= '0';
				sigmoid_enable_i <= '1';
			when others =>
				switch_enable_i <= '0';
				sigmoid_enable_i <= '0';
		end case;
	end process;
	
	NEXT_STATE_DECODE: process(state, enable)
	begin
	
		next_state <= state;
		
		case (state) is
			when st1_idle =>
				if(enable = '1') then
					next_state <= st2_read;
				end if;
			when st2_read =>
				if(enable = '0') then
					next_state <= st3_done;
				end if;
			when st3_done =>
				next_state <= st3_done;
			when others =>
				next_state <= st1_idle;
		end case;
	end process;

end Behavioral;

